#!/usr/bin/ruby
require "cgi"
require "net/http"
require "uri"
require "json"
require "digest/md5"

API_KEY=''
UNITS='miles' # miles or km
SOX='sox'
FAAC='faac'
PERL='perl'
SAY='say'
WGET='wget'
CHAPTERTOOL='/Applications/GarageBand.app/Contents/MacOS/ChapterTool'

def self.makeChapter( text, link, image, args )
  #MD5 the text and use for filename
  say_text = text
  if say_text == "Introduction"
    say_text = "Multi map.com Travel Directions Podcast"
  end
  if say_text == "Entering Congestion Charging"
    say_text = "You have now entered a congestion charging area"
  end
  if say_text == "Exiting Congestion Charging"
    say_text = "You have now left a congestion charging area"
  end
  md5 = Digest::MD5::hexdigest( say_text )
  # The 'say' command outputs AIFF
  aiff_filename = "snippets/" + md5 + ".aiff"
  # The getfilelength.pl command requires WAV
  wav_filename =  "snippets/" + md5 + ".wav"
  # To concatenate all the snippets, we need RAW
  raw_filename =  "snippets/" + md5 + ".raw"
  if File.exists? raw_filename
  else
    `#{SAY} -o #{aiff_filename} "#{say_text}"`
    `#{SOX} #{aiff_filename} #{wav_filename}`
    `#{SOX} #{aiff_filename} -r 22050 -c 1 -s -w #{raw_filename}`
  end
  length = `#{PERL} getfilelength.pl #{wav_filename}`
  starttime = args[0]
  args[0] += length.to_f
  args[1] += " #{raw_filename}"
  <<EOXML
  <chapter starttime="#{starttime}">
    <title>#{text}</title>
    <picture>#{image}</picture>
    <link href="#{link}">Map</link>
  </chapter>
EOXML
end

def self.getImage( image )
  #MD5 the url and use for filename
  md5 = Digest::MD5::hexdigest( image )
  png_image = "Pictures/#{md5}.png"
  if File.exists? png_image
  else
    `#{WGET} '#{image}' -O #{png_image}`
  end
  png_image
end

if ARGV.length < 2
  print "usage: makeroutepodcast.rb <source> <destination>\n"
  print "note : source and destination should be entered as quick searches\n"
elsif API_KEY == ''
  print "You must sign up for a Multimap Open API key:\n"
  print "  http://www.multimap.com/openapi/\n"
else
  source = CGI::escape(ARGV[0])
  destination = CGI::escape(ARGV[1])
  xmlFile = File.new( "route.xml", "w" )

  xmlFile << <<EOXML
<?xml version="1.0" encoding="utf-8"?>
<chapters version="1">
EOXML
  
  path_str = '/API/route/1.2/'+API_KEY+'?output=json&qs_1='+source+'&qs_2='+destination
  print "Requesting - " + path_str +"\n"
  res = Net::HTTP.start('developer.multimap.com', 80) {|http|
    http.get(path_str)
  }
  json_str = res.body
  route = JSON.parse( json_str )
  if route.nil? || route["stages"].nil?
    print "There was a problem generating this route, here's the output from the Multimap API:"
    print route.inspect
    exit
  end
  print "This route has " + route["stages"][0]["steps"].length.to_s + " steps\n"
  link = "http://www.multimap.com/maps/#t=r&amp;map="+route["stages"][0]["steps"][0]["start_point"]["lat"]+","+route["stages"][0]["steps"][0]["start_point"]["lon"]
  link += "%7c16%7c4&amp;route="+route["key"]
  link += "%7c#{source}:#{destination}%7c#{source}:#{destination}"
  image = "http://developer.multimap.com/API/map/1.2/"+API_KEY+"?width=340&height=320&routeKey="+route["key"]+"&output=png"
  image = getImage( image )
  args = [0,""]
  xmlFile << makeChapter( "Introduction", link, image, args )
  route["stages"][0]["steps"].each do |step|
    instruction = step["instruction"]
    if step["road_name"]
      instruction += " "+step["road_name"]
      if step["road_number"]
        instruction += " ("+step["road_number"]+")"
      end
    else
      if step["road_number"]
        instruction += " "+step["road_number"]
      end
    end
    if step["instruction"] != "Arrive at your destination"
      instruction += " for "+step["distance"][UNITS].to_f.to_s+" "+UNITS
    end
    instruction += "."
    link = "http://www.multimap.com/maps/#t=r&amp;map="+step["start_point"]["lat"]+","+step["start_point"]["lon"]
    link += "%7c16%7c4&amp;route="+route["key"]
    link += "%7c#{source}:#{destination}%7c#{source}:#{destination}"
    image = "http://developer.multimap.com/API/map/1.2/"+API_KEY+"?output=png&width=340&height=320&routeKey="+route["key"]+"&bb="+step["start_point"]["lon"]+","+step["start_point"]["lat"]+";"+step["start_point"]["lon"]+","+step["start_point"]["lat"]+"&lon_1="+step["start_point"]["lon"]+"&lat_1="+step["start_point"]["lat"]+"&marker_1=circle&label_1=+"

    image = getImage( image )
    print instruction+"\n"
    xmlFile << makeChapter( instruction, link, image, args )
  end
  xmlFile << "</chapters>"
  xmlFile.close()
  `cat #{args[1]} > route.raw`
  `#{SOX} -r 22050 -c 1 -s -w route.raw route.wav`
  `#{FAAC} route.wav -o route.m4a`
  `#{CHAPTERTOOL} -x route.xml -a route.m4a -o route.m4b`
end
